@@ -116,7 +116,7 @@ static void games_card_images_purge (GamesCardImages * images)
 GdkPixbuf * games_card_images_get_card_by_id (GamesCardImages * images,
 					      gint cardid)
 {
-  g_return_if_fail ((cardid >= 0) && (cardid < GAMES_CARDS_TOTAL));
+  g_return_val_if_fail ((cardid >= 0) && (cardid < GAMES_CARDS_TOTAL), NULL);
 
   if (!images->prerendered)
     games_card_images_prerender (images);
@@ -130,10 +130,12 @@ GdkPixbuf * games_card_images_get_card_by_id (GamesCardImages * images,
 GdkPixbuf * games_card_images_get_card (GamesCardImages * images, gint suit,
 					gint rank)
 {
-  g_return_if_fail ((suit >= GAMES_CARDS_CLUBS) && 
-		    (suit <= GAMES_CARDS_SPADES));
-  g_return_if_fail ((rank >= GAMES_CARD_ACE) && 
-		    (rank <= GAMES_CARD_ACE_HIGH));
+  g_return_val_if_fail (((suit >= GAMES_CARDS_CLUBS) && 
+			 (suit <= GAMES_CARDS_SPADES)),
+			NULL);
+  g_return_val_if_fail (((rank >= GAMES_CARD_ACE) && 
+			 (rank <= GAMES_CARD_ACE_HIGH)),
+			NULL);
 
   return games_card_images_get_card_by_id (images, GAMES_CARD_ID(suit, rank));
 }
